@@ -38,7 +38,9 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <sys/types.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
+#include <sys/resource.h>
 #include <signal.h>
 #include <pwd.h>
 #include <grp.h>
@@ -49,6 +51,7 @@
 #include "vicious.h"
 
 #include "gdm.h"
+#include "gdmconfig.h"
 #include "verify.h"
 #include "gdm-net.h"
 #include "misc.h"
@@ -135,6 +138,7 @@ static gchar *GdmSoundOnLoginFile = NULL;
 static gchar *GdmSoundOnLoginSuccessFile = NULL;
 static gchar *GdmSoundOnLoginFailureFile = NULL;
 static gchar *GdmConsoleCannotHandle = NULL;
+static gchar *GdmPamStack = NULL;
 
 static gint GdmXineramaScreen = 0;
 static gint GdmUserMaxFile = 0;
@@ -150,6 +154,7 @@ static gint GdmRetryDelay = 0;
 static gint GdmTimedLoginDelay = 0;
 static gint GdmFlexibleXservers = 5;
 static gint GdmFirstVt = 7;
+static gint GdmXserverTimeout = 10;
 
 /* The SDTLOGIN feature is Solaris specific, and causes the Xserver to be
  * run with user permissionsinstead of as root, which adds security but
@@ -502,6 +507,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_HOSTS, &GdmHosts, &string_type);
    gdm_config_add_hash (GDM_KEY_PRE_FETCH_PROGRAM,
       &GdmPreFetchProgram, &string_type);
+   gdm_config_add_hash (GDM_KEY_PAM_STACK, &GdmPamStack, &string_type);
 
    /* int values */
    gdm_config_add_hash (GDM_KEY_XINERAMA_SCREEN, &GdmXineramaScreen, &int_type);
@@ -532,6 +538,7 @@ gdm_config_init (void)
       &GdmBackgroundProgramInitialDelay, &int_type);
    gdm_config_add_hash (GDM_KEY_BACKGROUND_PROGRAM_RESTART_DELAY,
       &GdmBackgroundProgramRestartDelay, &int_type);
+   gdm_config_add_hash (GDM_KEY_XSERVER_TIMEOUT, &GdmXserverTimeout, &int_type);
 }
 
 /**
@@ -606,6 +613,19 @@ gdm_get_custom_config (struct stat *statbuf)
    }
 }
 
+/**
+ * gdm_get_per_display_custom_config_file
+ *
+ * Returns the per-display config file for a given display
+ * This is always the custom config file name with the display
+ * appended, and never gdm.conf.
+ */
+gchar *
+gdm_get_per_display_custom_config_file (gchar *display)
+{
+  g_strdup_printf ("%s%s", custom_config_file, display);
+}
+ 
 /**
  * gdm_get_custom_config_file
  *
@@ -701,6 +721,171 @@ gdm_get_value_bool (char *key)
    return FALSE;
 }
 
+/**
+ * Note that some GUI configuration parameters are read by the daemon,
+ * and in order for them to work, it is necessary for the daemon to 
+ * access a few keys in a per-display fashion.  These access functions
+ * allow the daemon to access these keys properly.
+ */
+
+/**
+ * gdm_get_value_int_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.
+ */
+gint gdm_get_value_int_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      gint val = atoi (perdispval);
+      g_free (perdispval);
+      return val;
+   } else {
+      return (gdm_get_value_int (key));
+   }
+}
+ 
+/**
+ * gdm_get_value_bool_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.
+ */
+gboolean gdm_get_value_bool_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      if (perdispval[0] == 'T' ||
+          perdispval[0] == 't' ||
+          perdispval[0] == 'Y' ||
+          perdispval[0] == 'y' ||
+          atoi (perdispval) != 0) {
+		g_free (perdispval);
+		return TRUE;
+       } else {
+		return FALSE;
+       }
+   } else {
+      return (gdm_get_value_bool (key));
+   }
+}
+ 
+/**
+ * gdm_get_value_string_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.  Note that this value needs to be freed,
+ * unlike the non-per-display version.
+ */
+gchar * gdm_get_value_string_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      return perdispval;
+   } else {
+      return (g_strdup (gdm_get_value_string (key)));
+   }
+}
+
+/**
+ * gdm_config_key_to_string_per_display
+ *
+ * If the key makes sense to be per-display, return the value,
+ * otherwise return NULL.  Keys that only apply to the daemon
+ * process do not make sense for per-display configuration  
+ * Valid keys include any key in the greeter or gui categories,
+ * and the GDM_KEY_PAM_STACK key.
+ *
+ * If additional keys make sense for per-display usage, make
+ * sure they are added to the if-test below.
+ */
+void
+gdm_config_key_to_string_per_display (gchar *display, gchar *key, gchar **retval)
+{
+   gchar *file;
+   gchar **splitstr = g_strsplit (key, "/", 2);
+   *retval = NULL;
+
+   if (display == NULL)
+      return;
+
+   file = gdm_get_per_display_custom_config_file (display);
+
+   if (strcmp (ve_sure_string (splitstr[0]), "greeter") == 0 ||
+       strcmp (ve_sure_string (splitstr[0]), "gui") == 0 ||
+       is_key (key, GDM_KEY_PAM_STACK)) {
+      gdm_config_key_to_string (file, key, retval);
+   }
+
+   g_free (file);
+
+   return;
+}
+
+/**
+ * gdm_config_key_to_string
+ *
+ * Returns a specific key from the config file, or NULL if not found.
+ * Note this returns the value in string form, so the caller needs
+ * to parse it properly if it is a bool or int.
+ */
+void
+gdm_config_key_to_string (gchar *file, gchar *key, gchar **retval)
+{
+   VeConfig *cfg = ve_config_get (file);
+   GdmConfigType *type = gdm_config_hash_lookup (type_hash, key);
+   gchar **splitstr = g_strsplit (key, "/", 2);
+   *retval = NULL;
+
+   /* Should not fail, all keys should have a category. */
+   if (splitstr[0] == NULL)
+      return;
+
+   /* If file doesn't exist, then just return */
+   if (cfg == NULL)
+      return;
+
+   GList *list = ve_config_get_keys (cfg, splitstr[0]);
+   while (list != NULL) {
+      gchar *display_key     = (char *)list->data;
+      gchar *display_fullkey = g_strdup_printf ("%s/%s", splitstr[0], display_key);
+
+      if (is_key (key, display_fullkey)) {
+         gdm_debug ("Returning value for key <%s>\n", key);
+         if (*type == CONFIG_BOOL) {
+            gboolean value = ve_config_get_bool (cfg, key);
+            if (value)
+               *retval = g_strdup ("true");
+            else
+               *retval = g_strdup ("false");
+            return;
+         } else if (*type == CONFIG_INT) {
+            gint value = ve_config_get_int (cfg, key);
+            *retval = g_strdup_printf ("%d", value);
+            return;
+         } else if (*type == CONFIG_STRING) {
+            gchar *value = ve_config_get_string (cfg, key);
+            if (value != NULL)
+               *retval = g_strdup (value);
+            return;
+         }
+      }
+      g_free (display_fullkey);
+      list = list->next;
+   }
+   return;
+}
+
 /**
  * gdm_config_to_string
  *
@@ -708,9 +893,20 @@ gdm_get_value_bool (char *key)
  * GET_CONFIG socket command.
  */ 
 void
-gdm_config_to_string (gchar *key, gchar **retval)
+gdm_config_to_string (gchar *key, gchar *display, gchar **retval)
 {
-   GdmConfigType *type;
+   GdmConfigType *type = gdm_config_hash_lookup (type_hash, key);
+   *retval = NULL;
+
+   /*
+    * See if there is a per-display config file, returning that value
+    * if it exists.
+    */
+   if (display) {
+      gdm_config_key_to_string_per_display (display, key, retval);
+      if (*retval != NULL)
+         return;
+   }
 
    /* First look in translated_hash */
    if (translated_hash != NULL) {
@@ -721,9 +917,6 @@ gdm_config_to_string (gchar *key, gchar **retval)
       }
    }
 
-   type   = gdm_config_hash_lookup (type_hash, key);
-   *retval = NULL;
-
    if (type != NULL) {
       if (*type == CONFIG_BOOL) {
          gboolean value = gdm_get_value_bool (key);
@@ -882,8 +1075,8 @@ _gdm_set_value_string (gchar *key, gchar *value_in, gboolean doing_update)
        } else {
           gdm_info (_("%s: BaseXsession empty; using %s/gdm/Xsession"),
                       "gdm_config_parse",
-                      EXPANDED_SYSCONFDIR);
-          *setting = g_build_filename (EXPANDED_SYSCONFDIR,
+                      GDMCONFDIR);
+          *setting = g_build_filename (GDMCONFDIR,
                                        "gdm", "Xsession", NULL);
        }
 
@@ -2064,39 +2257,6 @@ gdm_set_high_display_num (gint val)
    high_display_num = val;
 }
 
-/**
- * gdm_print_config_option
- *
- * Called by gdm_print_all_config in a loop to print each key.
- * 
- */
-void
-gdm_print_config_option (gpointer key_in, gpointer value_in, gpointer data)
-{
-   gchar *key = (gchar *)key_in;
-   gchar *retval;
-
-   gdm_config_to_string (key, &retval);
-   if (retval != NULL) {
-      gdm_debug ("key is %s, value is %s\n", key, retval);
-      g_free (retval);
-   } else
-      gdm_debug ("key is %s, value is <NULL>\n", key);
-}
-
-/**
- * gdm_print_all_config
- *
- * Not used by GDM, but useful for debug purposes.
- */
-void
-gdm_print_all_config (void)
-{
-   gdm_config_parse ();
-
-   g_hash_table_foreach (type_hash, gdm_print_config_option, NULL);
-}
-
 /**
  * gdm_is_valid_key
  *
@@ -2193,7 +2353,7 @@ static gboolean
 is_in_trusted_pic_dir (const char *path)
 {
         /* our own pixmap dir is trusted */
-        if (strncmp (path, EXPANDED_PIXMAPDIR, sizeof (EXPANDED_PIXMAPDIR)) == 0)
+        if (strncmp (path, PIXMAPDIR, sizeof (PIXMAPDIR)) == 0)
                 return TRUE;
 
         return FALSE;
