@@ -109,6 +109,7 @@ gdm_config_add_hash (GHashTable *hash, gchar *key, gpointer value)
 	if (p != NULL)
 		*p = '\0';
 
+	/* Do not free the newkey since it is going into the hash. */
 	g_hash_table_insert (hash, newkey, value);
 }
 
@@ -125,15 +126,21 @@ gdm_config_get_result (gchar *key)
 	gchar *newkey  = g_strdup (key);
 	gchar *command = NULL;
 	gchar *result  = NULL;
+	static char *display = NULL;
 
 	g_strstrip (newkey);
 	p = strchr (newkey, '=');
 	if (p != NULL)
 		*p = '\0';
 
-	command = g_strdup_printf ("GET_CONFIG %s", newkey);
-	result = gdmcomm_call_gdm (command, NULL /* auth cookie */,
-		"2.13.0.1", comm_tries);
+	display = g_strdup (g_getenv ("DISPLAY"));
+	if (display == NULL)
+		command = g_strdup_printf ("GET_CONFIG %s", newkey);
+	else
+		command = g_strdup_printf ("GET_CONFIG %s %s", newkey, display);
+
+	result  = gdmcomm_call_gdm (command, NULL /* auth cookie */,
+	          "2.13.0.1", comm_tries);
 
 	g_free (command);
 	g_free (newkey);
@@ -303,7 +310,7 @@ gdm_config_get_xservers (gboolean flexible)
 static gchar *
 _gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean doing_translated)
 {
-	gchar **hashretval = NULL;
+	gchar *hashretval = NULL;
 	gchar *result = NULL;
 	gchar *temp;
 
@@ -313,7 +320,7 @@ _gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean
 	hashretval = gdm_config_hash_lookup (string_hash, key);
 
 	if (reload == FALSE && hashretval != NULL)
-		return *hashretval;
+		return hashretval;
 
 	result = gdm_config_get_result (key);
 
@@ -352,26 +359,21 @@ _gdm_config_get_string (gchar *key, gboolean reload, gboolean *changed, gboolean
 		g_free (result);
 
 	if (hashretval == NULL) {
-		gchar **charval = g_new0 (gchar *, 1);
-		*charval = temp;
-		gdm_config_add_hash (string_hash, key, charval);
 
 		if (changed != NULL)
 			*changed = TRUE;
 
-		return *charval;
+		gdm_config_add_hash (string_hash, key, temp);
 	} else {
 		if (changed != NULL) {
-			if (strcmp (ve_sure_string (*hashretval), temp) != 0)
+			if (strcmp (ve_sure_string (hashretval), temp) != 0)
 				*changed = TRUE;
 			else
 				*changed = FALSE;
 		}
-
-		g_free (*hashretval);
-		*hashretval = temp;
-		return *hashretval;
+		g_hash_table_replace (string_hash, key, temp);
 	}
+	return temp;
 }
 
 gchar *
@@ -397,32 +399,36 @@ static gchar *
 _gdm_config_get_translated_string (gchar *key, gboolean reload, gboolean *changed)
 {
         const GList *li;
-        char *dkey;
+        char *newkey;
         char *def;
 
 	/* Strip key */
-        dkey = g_strdup (key);
-        def = strchr (dkey, '=');
-        if (def != NULL) {
-                *def = '\0';
-                def++;
-        }
+	newkey = g_strdup (key);
+	def    = strchr (newkey, '=');
+	if (def != NULL)
+		*def = '\0';
 
-        for (li = ve_i18n_get_language_list ("LC_MESSAGES");
-             li != NULL;
-             li = li->next) {
-                gchar *full = g_strdup_printf ("%s[%s]", dkey, (char *)li->data);
+	for (li = ve_i18n_get_language_list ("LC_MESSAGES");
+	     li != NULL;
+	     li = li->next) {
+                gchar *full = g_strdup_printf ("%s[%s]", newkey, (char *)li->data);
 
 		/*
-		 * Pass FALSE for last argument so it doesn't print errors for
+		 * Pass TRUE for last argument so it doesn't print errors for
 		 * failing to find the key, since this is expected
 		 */
-	        gchar *val = _gdm_config_get_string (full, reload, changed, TRUE);
+		gchar *val = _gdm_config_get_string (full, reload, changed, TRUE);
 
-                if (val != NULL)
+		g_free (full);
+
+                if (val != NULL) {
+			g_free (newkey);
 			return val;
+		}
         }
 
+	g_free (newkey);
+
 	/* Print error if it fails this time */
 	return _gdm_config_get_string (key, reload, changed, FALSE);
 }
@@ -486,7 +492,7 @@ _gdm_config_get_int (gchar *key, gboolean reload, gboolean *changed)
 
 	if (hashretval == NULL) {
 		gint *intval = g_new0 (gint, 1);
-		*intval = temp;
+		*intval      = temp;
 		gdm_config_add_hash (int_hash, key, intval);
 
 		if (changed != NULL)
@@ -577,7 +583,7 @@ _gdm_config_get_bool (gchar *key, gboolean reload, gboolean *changed)
 
 	if (hashretval == NULL) {
 		gboolean *boolval = g_new0 (gboolean, 1);
-		*boolval = temp;
+		*boolval          = temp;
 		gdm_config_add_hash (bool_hash, key, boolval);
 
 		if (changed != NULL)
