@@ -160,7 +160,6 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 	
 	greeter_item_pam_set_user (buf);
-	greeter_item_ulist_enable ();
 	printf ("%c\n", STX);
 	fflush (stdout);
 	break;
@@ -179,7 +178,6 @@ greeter_ctrl_handler (GIOChannel *source,
 		greeter_probably_login_prompt = FALSE;
 	}
 	greeter_ignore_buttons (FALSE);
-	greeter_item_ulist_enable ();
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, TRUE);
 	g_free (tmp);
 	break;
@@ -320,6 +318,8 @@ greeter_ctrl_handler (GIOChannel *source,
 	printf ("%c\n", STX);
 	fflush (stdout);
 	greeter_ignore_buttons (FALSE);
+	greeter_item_ulist_enable ();
+
 	break;
 
     case GDM_QUIT:
@@ -579,8 +579,8 @@ verify_gdm_version (void)
       gdm_wm_focus_new_windows (TRUE);
     
       msg =  g_strdup_printf (_("The greeter version (%s) does not match the daemon "
-                                "version.  "
-                                "You have probably just upgraded GDM.  "
+                                "version. "
+                                "You have probably just upgraded GDM. "
                                 "Please restart the GDM daemon or the computer."),
                               VERSION);
     
@@ -1342,6 +1342,9 @@ main (int argc, char *argv[])
   gtk_window_move (GTK_WINDOW (window), gdm_wm_screen.x, gdm_wm_screen.y);
   gtk_widget_show_now (window);
 
+  greeter_item_ulist_unset_selected_user ();
+  greeter_item_ulist_enable ();
+
   /* can it ever happen that it'd be NULL here ??? */
   if G_UNLIKELY (window->window != NULL)
     {
